# -*- coding: utf-8 -*-
from sc.recipe.staticresources import Recipe
from shutil import rmtree
from tempfile import mkdtemp

import os
import pkg_resources
import unittest


class RecipeTestCase(unittest.TestCase):

    def setUp(self):
        test_dir = os.path.realpath(mkdtemp())
        for directory in ('bin', 'parts', 'eggs', 'develop-eggs'):
            os.makedirs('{0}/{1}'.format(test_dir, directory))

        self.buildout_options = {
            'buildout': {
                'bin-directory': '{0}/bin'.format(test_dir),
                'parts-directory': '{0}/parts'.format(test_dir),
                'python': 'buildout',
                'executable': '{0}/bin/python2.7'.format(test_dir),
                'directory': '{0}'.format(test_dir),
                'find-links': '',
                'allow-hosts': '*',
                'eggs-directory': '{0}/eggs'.format(test_dir),
                'develop-eggs-directory': '{0}/develop-eggs'.format(test_dir),
            },
        }
        self.test_dir = test_dir
        self.options = {
            'recipe': 'sc.recipe.staticresources',
            'name': 'my.package.name',
            'short_name': 'mypackagename',
        }
        self.name = 'static-resources'
        self.static_resources = self._get_recipe()

    def tearDown(self):  # noqa
        rmtree(self.test_dir)

    def _get_recipe(self, buildout_options=None, name=None, options=None):
        if buildout_options is None:
            buildout_options = self.buildout_options
        if name is None:
            name = self.name
        if options is None:
            options = self.options
        return Recipe(
            buildout_options,
            name,
            options,
        )

    def test_minimal_options(self):
        static_resources = self._get_recipe(None, None, None)
        self.assertTrue(static_resources)
        self.assertIn('directory', static_resources.options)
        directory = '{0}/webpack'.format(self.test_dir)
        self.assertEqual(static_resources.options['directory'], directory)
        self.assertIn('version', static_resources.options)
        version = pkg_resources.get_distribution('sc.recipe.staticresources').version
        self.assertEqual(static_resources.options['version'], version)

    def test_custom_directory(self):
        options = self.options
        directory = '/tmp/webpack'
        options['directory'] = directory
        static_resources = self._get_recipe(None, None, options)
        self.assertEqual(static_resources.options['directory'], directory)
        static_resources.install()
        self.assertTrue(os.path.exists(directory))
        rmtree(directory)

    def test_custom_destination(self):
        options = self.options
        destination = '/tmp/static'
        options['destination'] = destination
        static_resources = self._get_recipe(None, None, options)
        self.assertEqual(static_resources.options['destination'], destination)
        self.static_resources.install()
        os.mkdir(destination)
        os.system('{0}/bin/build-mypackagename'.format(self.test_dir))
        self.assertTrue(os.path.exists(destination))
        rmtree(destination)

    def test_custom_bobtemplate(self):
        options = self.options
        bobtemplate = (
            'https://github.com/simplesconsultoria/sc.recipe.staticresources/'
            'archive/master.zip#sc.recipe.staticresources-master/src/sc/'
            'recipe/staticresources/bobtemplate'
        )
        options['bobtemplate'] = bobtemplate
        static_resources = self._get_recipe(None, None, options)
        self.assertEqual(static_resources.options['bobtemplate'], bobtemplate)
        static_resources.install()
        self.assertTrue(os.path.exists(options['directory']))

    def test_template_folder_being_created(self):
        self.static_resources.install()
        self.assertTrue(os.path.exists('{0}/webpack'.format(self.test_dir)))

    def test_remove_old_scritps(self):
        self.static_resources.install()
        self.assertTrue(os.path.exists('{0}/bin/build-mypackagename'.format(self.test_dir)))

        self.static_resources._remove_old_scritps()
        self.assertFalse(os.path.exists('{0}/bin/build-mypackagename'.format(self.test_dir)))



# -*- coding: utf-8 -*-
from glob import glob
from mrbob.configurator import parse_template
from mrbob.rendering import jinja2_renderer
from mrbob.rendering import render_structure
from os import path
from zc.buildout import UserError

import json
import logging
import os
import pkg_resources
import subprocess


VERSION = pkg_resources.get_distribution('sc.recipe.staticresources').version
CURRENT_DIR = path.dirname(__file__)
SCRIPT_TEMPLATE = """#!/bin/sh
export PATH={bin_directory}:$PATH
cd {webpack_directory}
yarn
{command}"""


class Recipe(object):

    """zc.buildout recipe"""

    def __init__(self, buildout, name, options):
        self.buildout = buildout
        self.name = name
        self.options = options
        self.logger = logging.getLogger(self.name)
        self.bin_directory = buildout['buildout']['bin-directory']

        # Check required options
        if not self.options.get('name', False):
            self._error('Missing "name" parameter with the package name.')
        if not self.options.get('short_name', False):
            self._error('Missing "short_name" parameter with a simplified name of package.')
        self.short_name = options['short_name']

        # Set default value for options
        directory = self.buildout['buildout']['directory']
        self.options.setdefault('directory', '{0}/webpack'.format(directory))
        self.webpack_directory = options['directory']
        self.options.setdefault('destination', 'dist')
        self.options.setdefault('bobtemplate', path.join(CURRENT_DIR, 'bobtemplate'))
        self.bobtemplate = options['bobtemplate']
        self.options.setdefault('version', VERSION)

    def _error(self, msg):
        self.logger.error(msg)
        raise UserError(msg)

    def _run_mrbob(self):
        template = parse_template(self.bobtemplate)[0]
        output_dir = self.webpack_directory
        variables = self.options
        verbose = True
        renderer = jinja2_renderer
        ignored_files = []
        ignored_directories = []

        os.mkdir(output_dir)
        render_structure(
            template,
            output_dir,
            variables,
            verbose,
            renderer,
            ignored_files,
            ignored_directories,
        )

    def _create_script(self, name, command):
        script_name = name + '-' + self.short_name
        out_file = self.bin_directory + '/' + script_name
        with open(out_file, 'w') as output:
            data = SCRIPT_TEMPLATE.format(
                bin_directory=self.bin_directory,
                webpack_directory=self.webpack_directory,
                command=command,
            )
            output.write(data)
        subprocess.call(['chmod', '775', out_file])
        self.logger.info('Install ' + script_name)

    def _remove_old_scritps(self):
        for script in glob(self.bin_directory + '/*-' + self.short_name):
            os.remove(script)

    def _create_new_scripts(self):
        with open(self.webpack_directory + '/package.json') as data_file:
            package_json = json.load(data_file)

        for name in package_json['scripts']:
            self._create_script(name, 'npm run ' + name)

        self._create_script('env', '/bin/bash')

        if 'build' in package_json['scripts']:
            subprocess.call(self.bin_directory + '/build-' + self.short_name)

    def install(self):
        # check if previous directory exists
        if not path.isdir(path.dirname(self.webpack_directory)):
            self.logger.error('Please check if this is a valid directory.')
            return

        if not path.isdir(self.webpack_directory):
            self._run_mrbob()

        self._remove_old_scritps()
        self._create_new_scripts()

    def update(self):
        self.install()



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '1.1b4'
description = 'FIXME'
long_description = (
    open('README.rst').read() + '\n' +
    open('CONTRIBUTORS.rst').read() + '\n' +
    open('CHANGES.rst').read()
)

entry_point = 'sc.recipe.staticresources:Recipe'
entry_points = {
    'zc.buildout': [
        'default = {0:s}'.format(entry_point)
    ]
}

setup(name='sc.recipe.staticresources',
      version=version,
      description=description,
      long_description=long_description,
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Framework :: Buildout',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python',
          'Topic :: Software Development :: Build Tools',
      ],
      keywords='',
      author='Simples Consultoria',
      author_email='products@simplesconsultoria.com.br',
      url='http://github.com/simplesconsultoria/sc.recipe.staticresources/',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['sc', 'sc.recipe'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'mr.bob',
          'setuptools',
          'zc.buildout',
          'zc.recipe.egg',
      ],
      extras_require={
          'test': [
              'testfixtures',
              'zc.buildout [test]',
              'zope.testing',
          ],
      },
      entry_points=entry_points,
      )
